import logging
from threading import Thread
from time import sleep

logger = logging.getLogger(__name__)


class EventListener():
    def __init__(self, event, handler, poll_interval, opts={}):
        logger.info(f'adding listener: {event.__name__}, handler: {handler.__name__}, poll_interval: {poll_interval}')
        self.filter = event.createFilter(fromBlock=1)
        self.poll_interval = poll_interval
        self.handler = handler

    def loop(self):
        while True:
            for event in self.filter.get_new_entries():
                self.handler(event)
            sleep(self.poll_interval)

    def run(self):
        self.worker = Thread(target=self.loop)
        self.worker.start()

    def stop(self):
        self.worker.join()



from skale.contracts import BaseContract
import skale.utils.helper as Helper


class Token(BaseContract):
    # def __init__(self, skale):
    #  name = 'token'
    #  super().__init__(skale, name, skale.abi[f'{name}_address'], skale.abi[f'{name}_abi'] )

    def test(self):
        pass



import logging

import skale.utils.helper as Helper
from web3 import Web3, WebsocketProvider
import skale.contracts as contracts

logger = logging.getLogger(__name__)


class Skale:
    def __init__(self, ip, port, abi_filepath=None):
        logger.info(f'Init skale-py: {ip}:{port}, custom ABI: {True if abi_filepath else False}')
        ws_addr = Helper.generate_ws_addr(ip, port)
        self.web3 = Web3(WebsocketProvider(ws_addr))
        self.abi = Helper.get_abi(abi_filepath)
        self.__contracts = {}
        self.__init_contracts()

    def __init_contracts(self):
        self.add_lib_contract('manager', contracts.NodeManager)
        self.add_lib_contract('token', contracts.Token)

        self.add_lib_contract('nodes', contracts.BaseContract)
        self.add_lib_contract('groups', contracts.BaseContract)
        self.add_lib_contract('validators', contracts.BaseContract)
        self.add_lib_contract('schains', contracts.BaseContract)

    def add_lib_contract(self, name, contract_class):
        address = self.abi[f'skale_{name}_address']
        abi = self.abi[f'skale_{name}_abi']

        self.add_contract(name, contract_class(self, name, address, abi))

    def add_contract(self, name, skale_contract):
        logger.debug(f'Init contract: {name}')
        self.__contracts[name] = skale_contract

    def get_contract_by_name(self, name):
        return self.__contracts[name]

    def __getattr__(self, name):
        if name not in self.__contracts:
            raise AttributeError(name)
        return self.get_contract_by_name(name)
